-> # Hello, Lua!

-> Pierre Chapuis
-> 2016-02-19, Lima ∞ Technology


-> Those slides are a support for live examples.
-> The goal is to introduce a few useful concepts
-> frequently found in Lua.

-> Prerequisite: having already written some code
-> in a dynamic language such as JavaScript,
-> Python on Ruby.


-> Note: those slides are made for [mdp](https://github.com/visit1985/mdp)
-> and fit in 54x24.

---

# Plan

- Chosen concepts
    - Types
    - Functions
    - Iterators
    - Tables and Metatables
    - Error handling
    - Coroutines

- Useful libraries and tools

- Resources and code bases to read

---

# Types

- nil
- boolean
- string
- number
^
- function
- table
^
- thread
^
- userdata

---

# Functions

## Variable arguments, multiple return

    dot = function(...)
        return ".", ...
    end

^
## First-class values and closures

    wrap = function(f)
        return function(...)
            print("before")
            f(...)
            print("after")
        end
    end

---

# Functions

## Proper tail calls

    t = {}
    for i=1, 3 do
        t[i] = function()
            print(i)
            io.read()
            return t[i % 3 + 1]()
        end
    end

---

# Functions

## Scope

    do -- just for the interpreter
        local i = 42
        local Counter = function()
            local i = 3
            do i = 6 end
            do local i = 9 end
            return function()
                print(i)
                i = i + 1
            end
        end
        i = 12
        local c = Counter()
        c()
        i = 15
        c()
    end

---

# Iterators

## Simple (closure)

    _ipairs = function(t)
        local i = 0
        return function()
            i = i + 1
            return t[i] and i, t[i]
        end
    end

^
## Stateless

    _next = function(inv, i)
        i = i + 1
        return inv[i] and i, inv[i]
    end
    _ipairs = function(t) return _next, t, 0 end

---

# Tables

## Basics

    t = {
        1, 2,
        f = 5,
        ["g"] = 6,
        [8] = {},
    }

^
## Methods

    o = {
        name = "joe",
        greet = function(self)
            print("hello, " .. self.name)
        end
    }

    o:greet()

---

# Tables

## Metatables

    mt = { __index = function(t, k) return 42 end }
    t = setmetatable({}, mt)

^
    mt = {
        __index = {
            name = "anonymous",
            greet = function(self)
                print("hello, " .. self.name)
            end
        }
    }

^
    new = function() return setmetatable({}, mt) end

---

# Tables

## Environnement

    do
        local _ENV = {}
        print("hello")
    end

^
    do
        local _ENV = {print = print}
        print("hello")
    end

^
    do
        local print = print
        local _ENV = {}
        print("hello")
    end

---

# Error handling

Use `error()` or `assert()` to indicate
a misuse of your API or a bug. [1]

Return `nil, emsg, ...` to indicate
an expected error (e.g. I/O). [2]

[2] -> [1] : `assert()`

[1] -> [2] : `pcall()`

---

# Coroutines

    co = coroutine
    c = co.create(function(x)
        while true do
            x = x + 1
            local y = co.yield(x)
            if y > 10 then return x end
        end
    end)
    co.resume(c, 3)

---

# Tools

- [LuaRocks](https://luarocks.org/)
    - [localua](https://github.com/catwell/localua), [hererocks](https://github.com/mpeterv/hererocks), [luawinmulti](https://github.com/Tieske/luawinmulti)

^
- [Penlight](https://github.com/stevedonovan/Penlight)
    - `pl.strict`, `pl.pretty`

^
- [LuaSocket](http://w3.impa.br/~diego/software/luasocket/)

^
- Web Frameworks
    - [Sailor](http://sailorproject.org/), [Lapis](http://leafo.net/lapis/)

---

# Tools

- Tests
    - [Busted](https://github.com/Olivine-Labs/busted), [cwtest](https://github.com/catwell/cwtest)

^
- [luacheck](https://github.com/mpeterv/luacheck)
    - Plugins for: Vim, Emacs,  Sublime, Atom

^
- [MobDebug](https://github.com/pkulchenko/MobDebug) and [ZeroBrane Studio](https://studio.zerobrane.com/)

^
- [lmprof](https://github.com/pmusa/lmprof)

^
- [luabuild](https://github.com/stevedonovan/luabuild) and soar

---

# Resources

- [Programming in Lua](http://www.lua.org/pil/), *3rd edition*
    - Available from me in paper format

- [Le guide de Lua et ses applications](http://www.d-booker.fr/programmation-et-langage/1-livre-lua.html)
    - Available from me (V2 unofficial WIP PDF)

- [Lua reference Manual](http://www.lua.org/manual/5.3/manual.html)

- [Lua Unofficial FAQ](http://www.luafaq.org/), [Lua.Space](http://lua.space/)

- [Lua Users wiki](http://lua-users.org/wiki/) (warning: outdated material)

---

# Code bases

- [middleclass](https://github.com/kikito/middleclass): a class library, for metatables
^
- [copas](https://github.com/keplerproject/copas): a scheduler, for coroutines

^
- [luagleck](https://github.com/ignacio/luagleck)
- [LuaRocks](https://github.com/keplerproject/luarocks)
- [Prosody](https://github.com/bjc/prosody)
- [TextAdept](https://github.com/rgieseke/textadept)
- [lunamark](https://github.com/jgm/lunamark)
- [CorsixTH](https://github.com/CorsixTH/CorsixTH)

---

-> # That's All Folks!

-> Thank you for listening.

-> Questions?
